@@ -550,9 +550,9 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
     Emit a code. \
   */ \
   if (bits > 0) \
-    datum|=(code) << bits; \
+    datum|=(size_t) (code) << bits; \
   else \
-    datum=code; \
+    datum=(size_t) (code); \
   bits+=number_bits; \
   while (bits >= 8) \
   { \
@@ -715,7 +715,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
           if (next_pixel != MagickFalse)
             continue;
         }
-      GIFOutputCode((size_t) waiting_code);
+      GIFOutputCode(waiting_code);
       if (free_code < MaxGIFTable)
         {
           hash_code[k]=(short) free_code++;
@@ -785,7 +785,7 @@ static MagickBooleanType EncodeImage(const ImageInfo *image_info,Image *image,
   /*
     Flush out the buffered code.
   */
-  GIFOutputCode((size_t) waiting_code);
+  GIFOutputCode(waiting_code);
   GIFOutputCode(end_of_information_code);
   if (bits > 0)
     {
@@ -991,10 +991,10 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   unsigned char
     background,
+    buffer[257],
     c,
     flag,
     *global_colormap,
-    header[MaxTextExtent],
     magick[12];
 
   /*
@@ -1021,6 +1021,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((count != 6) || ((LocaleNCompare((char *) magick,"GIF87",5) != 0) &&
       (LocaleNCompare((char *) magick,"GIF89",5) != 0)))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  (void) ResetMagickMemory(buffer,0,sizeof(buffer));
   page.width=ReadBlobLSBShort(image);
   page.height=ReadBlobLSBShort(image);
   flag=(unsigned char) ReadBlobByte(image);
@@ -1080,11 +1081,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             /*
               Read graphics control extension.
             */
-            while (ReadBlobBlock(image,header) != 0) ;
-            dispose=(size_t) (header[0] >> 2);
-            delay=(size_t) ((header[2] << 8) | header[1]);
-            if ((ssize_t) (header[0] & 0x01) == 0x01)
-              opacity=(ssize_t) header[3];
+            while (ReadBlobBlock(image,buffer) != 0) ;
+            dispose=(size_t) buffer[0] >> 2;
+            delay=((size_t) buffer[2] << 8) | buffer[1];
+            if ((ssize_t) (buffer[0] & 0x01) == 0x01)
+              opacity=(ssize_t) buffer[3];
             break;
           }
           case 0xfe:
@@ -1101,11 +1102,11 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
             comments=AcquireString((char *) NULL);
             for (length=0; ; length+=count)
             {
-              count=ReadBlobBlock(image,header);
+              count=ReadBlobBlock(image,buffer);
               if (count == 0)
                 break;
-              header[count]='\0';
-              (void) ConcatenateString(&comments,(const char *) header);
+              buffer[count]='\0';
+              (void) ConcatenateString(&comments,(const char *) buffer);
             }
             (void) SetImageProperty(meta_image,"comment",comments);
             comments=DestroyString(comments);
@@ -1120,13 +1121,13 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
               Read Netscape Loop extension.
             */
             loop=MagickFalse;
-            if (ReadBlobBlock(image,header) != 0)
-              loop=LocaleNCompare((char *) header,"NETSCAPE2.0",11) == 0 ?
+            if (ReadBlobBlock(image,buffer) != 0)
+              loop=LocaleNCompare((char *) buffer,"NETSCAPE2.0",11) == 0 ?
                 MagickTrue : MagickFalse;
             if (loop != MagickFalse)
-              while (ReadBlobBlock(image,header) != 0)
+              while (ReadBlobBlock(image,buffer) != 0)
               {
-                iterations=(size_t) ((header[2] << 8) | header[1]);
+                iterations=((size_t) buffer[2] << 8) | buffer[1];
                 if (iterations != 0)
                   iterations++;
               }
@@ -1155,13 +1156,13 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 /*
                   Store GIF application extension as a generic profile.
                 */
-                icc=LocaleNCompare((char *) header,"ICCRGBG1012",11) == 0 ?
+                icc=LocaleNCompare((char *) buffer,"ICCRGBG1012",11) == 0 ?
                   MagickTrue : MagickFalse;
-                magick=LocaleNCompare((char *) header,"ImageMagick",11) == 0 ?
+                magick=LocaleNCompare((char *) buffer,"ImageMagick",11) == 0 ?
                   MagickTrue : MagickFalse;
-                i8bim=LocaleNCompare((char *) header,"MGK8BIM0000",11) == 0 ?
+                i8bim=LocaleNCompare((char *) buffer,"MGK8BIM0000",11) == 0 ?
                   MagickTrue : MagickFalse;
-                iptc=LocaleNCompare((char *) header,"MGKIPTC0000",11) == 0 ?
+                iptc=LocaleNCompare((char *) buffer,"MGKIPTC0000",11) == 0 ?
                   MagickTrue : MagickFalse;
                 number_extensionss++;
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -1171,6 +1172,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (info == (unsigned char *) NULL)
                   {
                     meta_image=DestroyImage(meta_image);
+                    global_colormap=(unsigned char *) RelinquishMagickMemory(
+                      global_colormap);
                     ThrowReaderException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
@@ -1189,6 +1192,9 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                       if (info == (unsigned char *) NULL)
                         {
                           meta_image=DestroyImage(meta_image);
+                          global_colormap=(unsigned char *)
+                            RelinquishMagickMemory(global_colormap);
+                          info=(unsigned char *) RelinquishMagickMemory(info);
                           ThrowReaderException(ResourceLimitError,
                             "MemoryAllocationFailed");
                         }
@@ -1198,6 +1204,8 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if (profile == (StringInfo *) NULL)
                   {
                     meta_image=DestroyImage(meta_image);
+                    global_colormap=(unsigned char *) RelinquishMagickMemory(
+                      global_colormap);
                     ThrowReaderException(ResourceLimitError,
                       "MemoryAllocationFailed");
                   }
@@ -1215,7 +1223,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 else
                   (void) FormatLocaleString(name,sizeof(name),"gif:%.11s",
-                    header);
+                    buffer);
                 info=(unsigned char *) RelinquishMagickMemory(info);
                 if (magick == MagickFalse)
                   (void) SetImageProfile(meta_image,name,profile);
@@ -1227,7 +1235,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           default:
           {
-            while (ReadBlobBlock(image,header) != 0) ;
+            while (ReadBlobBlock(image,buffer) != 0) ;
             break;
           }
         }
@@ -1270,7 +1278,7 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     image->colors=BitSet((int) flag,0x80) == 0 ? global_colors : one <<
       ((size_t) (flag & 0x07)+1);
     if (opacity >= (ssize_t) image->colors)
-      opacity=(-1);
+      image->colors=opacity+1;
     image->page.width=page.width;
     image->page.height=page.height;
     image->page.y=page.y;
@@ -1371,6 +1379,9 @@ static Image *ReadGIFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     status=SetImageExtent(image,image->columns,image->rows);
     if (status == MagickFalse)
       {
+        global_colormap=(unsigned char *) RelinquishMagickMemory(
+          global_colormap);
+        meta_image=DestroyImage(meta_image);
         InheritException(exception,&image->exception);
         return(DestroyImageList(image));
       }
@@ -1760,7 +1771,8 @@ static MagickBooleanType WriteGIFImage(const ImageInfo *image_info,Image *image)
             (void) WriteBlob(image,11,(unsigned char *) "NETSCAPE2.0");
             (void) WriteBlobByte(image,(unsigned char) 0x03);
             (void) WriteBlobByte(image,(unsigned char) 0x01);
-            (void) WriteBlobLSBShort(image,(unsigned short) (image->iterations ? image->iterations - 1 : 0));
+            (void) WriteBlobLSBShort(image,(unsigned short) (image->iterations ?
+              image->iterations-1 : 0));
             (void) WriteBlobByte(image,(unsigned char) 0x00);
           }
         if ((image->gamma != 1.0f/2.2f))
